// Copyright 2009 Green Code LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef MCC_PROGRAM_EXCEPTIONS_H
#define MCC_PROGRAM_EXCEPTIONS_H

#include <exception>
#include <string>

#include <boost/filesystem/path.hpp>

namespace mcc
{
  // Base class for all the exceptions generated by this program.
  struct ProgramException : public std::exception
  {
    std::string description;

    ProgramException(const std::string & description_)
      : description(description_)
    {
    }

    const char * what() const throw()
    {
      return description.c_str();
    }
    
    ~ProgramException() throw()
    {
    }
  };

  //---------------------------------------------------------------------------

  // Exception with the program's parameters.
  struct ProgramParameterException : public ProgramException
  {
    ProgramParameterException(const std::string & description)
      : ProgramException(description)
    {
    }
  };

  //---------------------------------------------------------------------------

  // Exception when opening a file.
  struct OpenFileException : public ProgramException
  {
    boost::filesystem::path path;

    OpenFileException(const boost::filesystem::path & path_,
                      const std::string &             description)
      : ProgramException(description), path(path_)
    {
    }
    
    ~OpenFileException() throw()
    {
    }
  };
}

#endif
